#include "DataStructures.h"
#include <iomanip>
#include <set>

std::ostream& operator<<(std::ostream& out_stream, const Node& node_)
{
	out_stream << std::setw(4) << node_.nodeID << " |"
		<< std::right << std::setw(11) << node_.cords[0] << std::setw(11) << node_.cords[1] << std::setw(11) << node_.cords[2]
		<< " | " << node_.vertex << "\n";
	return out_stream;
}

std::ostream& operator<<(std::ostream& out_stream, const std::vector<Node>& nodes_)
{
	out_stream << nodes_.size() << (nodes_.size() > 1 ? " Nodes:\n" : " Node:\n") << " ID"
		<< std::setw(10) << "X" << std::setw(11) << "Y" << std::setw(11) << "Z" << std::setw(14) << "Vertex\n";
	for (const auto& it : nodes_)
		out_stream << it;
	return out_stream;
}

std::ostream& operator<<(std::ostream& out_stream, const FiniteElement& element_) 
{
    out_stream << std::setw(4) << element_.elementID << " |"
        << std::setw(11) << element_.materialID << " |";
    for (const auto& nodes_ID_it : element_.nodesID)
        out_stream << std::right << std::setw(4) << nodes_ID_it;
    out_stream << "\n";
    return out_stream;
}


std::ostream& operator<<(std::ostream& out_stream, const std::vector<FiniteElement>& elements_)
{
	int elements_size = elements_.size();
	out_stream << elements_size << (elements_size > 1 ? " Elements:\n" : " Element:\n")
		<< " ID" << std::setw(15) << "Material_ID" << std::setw(15) << "Nodes ID\n";
	for (const auto& it : elements_)
		out_stream << it;
	return out_stream;
}

std::ostream& operator<<(std::ostream& out_stream, const BoundaryFiniteElement& surface_) 
{
    out_stream << std::setw(9) << surface_.surfaceID << " |"
        << std::setw(9) << surface_.boundaryID << " |";
    for (const auto& nodes_ID_it : surface_.nodesID)
        out_stream << std::right << std::setw(4) << nodes_ID_it;
    out_stream << "\n";
    return out_stream;
}


std::ostream& operator<<(std::ostream& out_stream, const std::vector<BoundaryFiniteElement>& boundaryFEs_)
{
	int boundaryFEs_size = boundaryFEs_.size();
	out_stream << boundaryFEs_size << (boundaryFEs_size > 1 ? " boundaryFEs:\n" : " Surface:\n")
		<< "Surface_ID" << std::setw(11) << "Border_ID" << std::setw(14) << "Nodes ID\n";
	for (const auto& it : boundaryFEs_)
		out_stream << it;
	return out_stream;
}
